from __future__ import unicode_literals

from django.urls import reverse

from contact_form.tests.base import ContactFormCase
from contact_form.forms.uniform import ContactForm


class TestContactForm(ContactFormCase):

    def test_form_view(self):
        response = self.client.get(reverse('contact_form:contact_form'))
        self.assertEqual(response.status_code, 200)

    def test_valid_form(self):
        data = dict(
            sender_name='test',
            sender_email='test@test.com',
            subject=self.subject_foo.pk,
            message='test message from user foo',
            captcha='',
        )
        form = ContactForm(data=data)
        self.assertTrue(form.is_valid())

    def test_invalid_form(self):
        data = dict(
            sender_name='test',
            sender_email='test@test.com',
            subject='',
            message=''
        )
        form = ContactForm(data=data)
        self.assertFalse(form.is_valid())



# from modeltranslation.translator import translator, TranslationOptions
# from contact_form.models import Subject


class SubjectTranslationOptions:
    fields = ('title', 'description')
    fallback_languages = {'default': ('en',)}


# translator.register(Subject, SubjectTranslationOptions)



from calendar import monthrange
from datetime import datetime, date

from django import forms

from .widgets import PartialDatePickerInput


class PartialDateField(forms.DateField):

    widget = PartialDatePickerInput

    def __init__(self, *, input_formats=None, **kwargs):
        super().__init__(**{**kwargs, 'widget': PartialDateField.widget})
        if input_formats is not None:
            self.input_formats = input_formats

    @staticmethod
    def _int(value: str) -> int:
        if '_' in value:
            return 0
        return int(value)

    def strptime(self, value, format):
        if '_' in value: # Only %d.%m.%Y format supported
            if format != '%d.%m.%Y':
                raise BaseException(f'The date format \'{format}\' is not supported. Please, try to implement it.')
            iso_value = value.split('.')
            day = iso_value[0] if len(iso_value) > 0 else '__'
            month = iso_value[1] if len(iso_value) > 1 else '__'
            year = iso_value[2] if len(iso_value) > 2 else '____'
            return f'{year}-{month}-{day}'
        else:
            return datetime.strptime(value, format).date().isoformat()

    def prepare_value(self, value: str):
        if value is None:
            return value
        date = value.split('-')
        year = date[0] if len(date) > 0 else '____'
        month = date[1] if len(date) > 1 else '__'
        day = date[2] if len(date) > 2 else '__'
        return f'{day}.{month}.{year}'


from calendar import monthrange
from datetime import datetime, date

from django import forms
from djeu.utils.jdcal import jcal2jd, jd2gcal, gcal2jd, jd2jcal

from .widgets import JulianDatePickerInput


class JulianDateField(forms.DateField):

    widget = JulianDatePickerInput

    def __init__(self, *, input_formats=None, **kwargs):
        super().__init__(**{**kwargs, 'widget': JulianDateField.widget})
        if input_formats is not None:
            self.input_formats = input_formats

    @staticmethod
    def _int(value: str) -> int:
        if '_' in value:
            return 0
        return int(value)

    def strptime(self, value, format):
        formated_date = value
        if '(' in value:
            formated_date = value[:value.index('(')].strip()
        # Convert from julian to gregorian
        julian_day, julian_month, julian_year = [
            self._int(v) for v in formated_date.split('.')]
        year, month, day, _ = jd2gcal(
            *jcal2jd(julian_year, julian_month, julian_day))
        # super -> return datetime.strptime(value, format).date()s
        return datetime(year, month, day)

    def prepare_value(self, value: datetime):
        if isinstance(value, date):
            julian_year, julian_month, julian_day, _ = jd2jcal(*gcal2jd(value.year, value.month, value.day))
            return f'{julian_day:02d}.{julian_month:02d}.{julian_year:04d} ({value.year:04d}-{value.month:02d}-{value.day:02d})'
        return value